/*
** nifconfig.c - New ifconfig
**
** Author: Peter Eriksson (pen@lysator.liu.se) 11 Feb 1991
*/

#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <netdb.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>

#include <net/if.h>
#include <netinet/in.h>

#define	IFFBITS \
"\020\1UP\2BROADCAST\3DEBUG\4ROUTE\5POINTOPOINT\6NOTRAILERS\7RUNNING\10NOARP\
"

struct ifreq	ifr;
int		s;
char		name[30];
int		flags;
int		metric;

int		debugflag = 0;


main(argc, argv)
  int argc;
  char *argv[];
{
  int	i;
  int opt_flags_flag = 0;
  int opt_addr_flag = 0;
  struct sockaddr opt_addr;
  int opt_dstaddr_flag = 0;
  struct sockaddr opt_dstaddr;
  int   opt_mtu = -1;
  

  if (argc < 2)
  {
    fprintf(stderr, "usage: nifconfig interface\n%s%s%s%s",
	    "\t[ address [ dest_addr ] [ up ] [ down ]\n",
	    "\t[ mtu <ddd> ]\n",
	    "\t[ trailers | -trailers ]\n",
	    "\t[ debug | -debug ]\n",
	    "\t[ arp | -arp ]\n");
    exit(1);
  }
  else
  {
    strncpy(name, argv[1], sizeof(name));
    strncpy(ifr.ifr_name, name, sizeof(ifr.ifr_name));
  }
  
  
  if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
  {
    perror("nifconfig: socket");
    exit(1);
  }

  if (ioctl(s, SIOCGIFFLAGS, (caddr_t)&ifr) < 0)
  {
    perror("ioctl (SIOCGIFFLAGS)");
    exit(1);
  }

  strncpy(ifr.ifr_name, name, sizeof ifr.ifr_name);
  flags = ifr.ifr_flags;

  if (argc == 2)
  {
    status();
    exit(0);
  }
  
  for (i = 2; i < argc; i++)
  {
    if (strcmp(argv[i], "up") == 0)
    {
      opt_flags_flag = 1;
      flags |= IFF_UP;
    }
    else if (strcmp(argv[i], "down") == 0)
    {
      opt_flags_flag = 1;
      flags &= ~IFF_UP;
    }
    else if (strcmp(argv[i], "mtu") == 0)
    {
      if (i+1 < argc && isascii(argv[++i][0]) && isdigit(argv[i][0]))
	opt_mtu = atoi(argv[i]);
      else
      {
	fprintf(stderr, "nifconfig: missing argument to 'mtu' option\n");
	exit(1);
      }
    }
    else if (strcmp(argv[i], "trailers") == 0)
    {
      opt_flags_flag = 1;
      flags &= ~IFF_NOTRAILERS;
    }
    else if (strcmp(argv[i], "-trailers") == 0)
    {
      opt_flags_flag = 1;
      flags |= IFF_NOTRAILERS;
    }
    else if (strcmp(argv[i], "arp") == 0)
    {
      opt_flags_flag = 1;
      flags &= ~IFF_NOARP;
    }
    else if (strcmp(argv[i], "-arp") == 0)
    {
      opt_flags_flag = 1;
      flags |= IFF_NOARP;
    }
    else if (strcmp(argv[i], "debug") == 0)
    {
      opt_flags_flag = 1;
      flags |= IFF_DEBUG;
    }
    else if (strcmp(argv[i], "-debug") == 0)
    {
      opt_flags_flag = 1;
      flags &= ~IFF_DEBUG;
    }
    else
    {
      if (!opt_addr_flag)
      {
	get_addr(argv[i], &opt_addr);
	opt_addr_flag = 1;
      }
      else if ((flags & IFF_POINTOPOINT) && !opt_dstaddr_flag)
      {
	get_addr(argv[i], &opt_dstaddr);
	opt_dstaddr_flag = 1;
      }
      else
      {
	fprintf(stderr, "nifconfig: unknown option: %s\n", argv[i]);
	exit(1);
      }
    }
  }

  if (opt_flags_flag)
  {
    puts("**** opt_flags ****");
    strncpy(ifr.ifr_name, name, sizeof ifr.ifr_name);
    ifr.ifr_flags = flags;
    if (ioctl(s, SIOCSIFFLAGS, (caddr_t) &ifr) < 0)
    {
      perror("nifconfig: ioctl(SIOCSIFFLAGS)");
      exit(1);
    }
  }
  
  if (opt_addr_flag)
  {
    strncpy(ifr.ifr_name, name, sizeof ifr.ifr_name);
    bcopy(&opt_addr, &ifr.ifr_addr, sizeof(struct sockaddr));
    if (ioctl(s, SIOCSIFADDR, (caddr_t) &ifr) < 0)
    {
      perror("nifconfig: ioctl(SIOCSIFADDR)");
      exit(1);
    }
  }

  if (opt_addr_flag && (flags & IFF_POINTOPOINT) && !opt_dstaddr_flag)
  {
    opt_dstaddr = opt_addr;
    opt_dstaddr_flag = 1;
  }
  
  if (opt_dstaddr_flag)
  {
    strncpy(ifr.ifr_name, name, sizeof ifr.ifr_name);
    bcopy(&opt_dstaddr, &ifr.ifr_dstaddr, sizeof(struct sockaddr));
    if (ioctl(s, SIOCSIFDSTADDR, (caddr_t) &ifr) < 0)
    {
      perror("nifconfig: ioctl(SIOCSIFDSTADDR)");
      exit(1);
    }
  }

  if (opt_mtu != -1)
  {
    strncpy(ifr.ifr_name, name, sizeof ifr.ifr_name);
    ifr.ifr_mtu = opt_mtu;
    if (ioctl(s, SIOCSIFMTU, (caddr_t) &ifr) < 0)
    {
      perror("nifconfig: ioctl(SIOCSIFMTU)");
      exit(1);
    }
  }
}


get_addr(s, sin)
  char *s;
  struct sockaddr_in *sin;
{
  struct in_addr inet_makeaddr();
  int val;
  struct hostent *hp;
  struct netent *np;
  
  sin->sin_family = AF_INET;
  if ((val = inet_addr(s)) != -1)
    sin->sin_addr.s_addr = val;
  else if (hp = gethostbyname(s))
    bcopy(hp->h_addr, (char *)&sin->sin_addr, hp->h_length);
  else if (np = getnetbyname(s))
    sin->sin_addr = inet_makeaddr(np->n_net, INADDR_ANY);
  else {
    fprintf(stderr, "nifconfig: %s: bad value\n", s);
    exit(1);
  }
}



/*
 * Print a value a la the %b format of the kernel's printf
 */
printb(s, v, bits)
  char *s;
  register char *bits;
  register unsigned short v;
{
  register int i, any = 0;
  register char c;

  if (bits && *bits == 8)
    printf("%s=%o", s, v);
  else
    printf("%s=%x", s, v);
  bits++;
  if (bits) {
    putchar('<');
    while (i = *bits++) {
      if (v & (1 << (i-1))) {
        if (any)
          putchar(',');
        any = 1;
        for (; (c = *bits) > 32; bits++)
          putchar(c);
      } else
        for (; *bits > 32; bits++)
          ;
    }
    putchar('>');
  }
}


in_status(force)
  int force;
{
  struct sockaddr_in *sin;
  char *inet_ntoa();
  
  strncpy(ifr.ifr_name, name, sizeof (ifr.ifr_name));
  if (ioctl(s, SIOCGIFADDR, (caddr_t)&ifr) < 0) {
    if (errno == EADDRNOTAVAIL || errno == EAFNOSUPPORT) {
      if (!force)
	return;
      bzero((char *)&ifr.ifr_addr, sizeof(ifr.ifr_addr));
    } else
      perror("ioctl (SIOCGIFADDR)");
  }
  sin = (struct sockaddr_in *)&ifr.ifr_addr;
  printf("\tinet %s ", inet_ntoa(sin->sin_addr));
  strncpy(ifr.ifr_name, name, sizeof (ifr.ifr_name));
  if (flags & IFF_POINTOPOINT) {
    if (ioctl(s, SIOCGIFDSTADDR, (caddr_t)&ifr) < 0) {
      if (errno == EADDRNOTAVAIL)
	bzero((char *)&ifr.ifr_addr, sizeof(ifr.ifr_addr));
      else
	perror("ioctl (SIOCGIFDSTADDR)");
    }
    strncpy(ifr.ifr_name, name, sizeof (ifr.ifr_name));
    sin = (struct sockaddr_in *)&ifr.ifr_dstaddr;
    printf("--> %s ", inet_ntoa(sin->sin_addr));
  }
  putchar('\n');

  if (ioctl(s, SIOCGIFMTU, (caddr_t)&ifr) < 0)
    perror("ioctl (SIOCGIFMTU)");
  else
    printf("\tmtu  %d\n", ifr.ifr_mtu);
}


status()
{
  printf("%s: ", name);
  printb("flags", flags, IFFBITS);
  putchar('\n');

  in_status(1);
}
